#include "io.h"
#include "lcd.h"
#include <avr/pgmspace.h>
#define F_CPU 8000000
#include <util/delay.h>
#include "font_8x8.c"
#include <stdio.h>

//LCD init sequence. Copypasted from somewhere, dunno where exactly anymore.
//First number in the line is the length of the command that follows,
//rest are the command bytes.
unsigned char lcdInitSeq[] PROGMEM={
	5, 0xf4, 0x90, 0xb3, 0xa0, 0xd0,
    14, 0xf0, 0xe2, 0xd4, 0x70, 0x66, 0xb2, 0xba, 0xa1, 0xa3, 0xab, 
          0x94, 0x95, 0x95, 0x95,
    2, 0xf5, 0x90,
    15, 0xf1, 0x00, 0x10, 0x22, 0x30, 0x45, 0x50, 0x68, 0x70, 0x8a,
          0x90, 0xac, 0xb0, 0xce, 0xd0,
    15, 0xf2, 0x0f, 0x10, 0x20, 0x30, 0x43, 0x50, 0x66, 0x70, 0x89, 
          0x90, 0xab, 0xb0, 0xcd, 0xd0,
    15, 0xf3, 0x0e, 0x10, 0x2f, 0x30, 0x40, 0x50, 0x64, 0x70, 0x87,
          0x90, 0xaa, 0xb0, 0xcb, 0xd0,
    7, 0xf4, 0x0d, 0x10, 0x2e, 0x30, 0x4f, 0x50,
    2, 0xf5, 0x91, 
    15, 0xf1, 0x01, 0x11, 0x22, 0x31, 0x43, 0x51, 0x64, 0x71, 0x86,
          0x91, 0xa8, 0xb1, 0xcb, 0xd1,
    15, 0xf2, 0x0f, 0x11, 0x21, 0x31, 0x42, 0x51, 0x63, 0x71, 0x85,
          0x91, 0xa6, 0xb1, 0xc8, 0xd1,
    15, 0xf3, 0x0b, 0x11, 0x2f, 0x31, 0x41, 0x51, 0x62, 0x71, 0x83,
          0x91, 0xa4, 0xb1, 0xc6, 0xd1,
    7, 0xf4, 0x08, 0x11, 0x2b, 0x31, 0x4f, 0x51,
    2, 0x80, 0x94,
    2, 0xf5, 0xa2,
    2, 0xf4, 0x60,
    4, 0xf0, 0x40, 0x50, 0xc0,
    2, 0xf4, 0x70,
    2, 0xf0, 0x81,
//    3, 0xf4, 0xb3, 0xa0,
    3, 0xf4, 0xb1, 0xac,
    5, 0xf0, 0x00 | 0x6, 0x10, 0x20, 0x30,
    5, 0xf5, 0x00 | 0xf, 0x10 | 0xc, 0x20 | 0xf, 0x30 | 0x4,
    0
};

char xpos;
char ypos;
short fgcol;
short bgcol;

//Send a pixel to the LCD. The color will be what's defined in
//fgColor when asFg is true, or bgColor when asFg is false.
void sendpixel(char asFg) {
	short c;
	if (asFg) c=fgcol; else c=bgcol;
	sendByteLcd(c&0xff);
	sendByteLcd(c>>8);
}

static int lcdPutchar(char c, FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(lcdPutchar, NULL, _FDEV_SETUP_WRITE);

//Putchar, for printf. Takes a character and looks up the corresponding image
//in the font, then puts that on the screen.
static int lcdPutchar(char c, FILE *stream) {
	char x,y, b;
	int pos;
	if (c>=32 && c<128) { //only print chars in the font
		pos=((c-32)*8);
		windowLcd(xpos,ypos,xpos+7,ypos+8);
		sendDataLcd();
		for (x=0; x<8; x++) {
			b=pgm_read_byte(&fontdata_8x8[pos++]);
			for (y=0; y<8; y++) {
				sendpixel(b&0x80);
				b<<=1;
			}
		}
		endCmdLcd();
		xpos+=8;
	}
	if (xpos>104-7 || c=='\n') {
		ypos+=8;
		xpos=3;
		if (ypos>=80) ypos=0;
	}
	return 1;
}

void initLcd() {
	int x=0;
	char ctr;
	resetLcd();
	//Send init sequence to lcd
	while(pgm_read_byte(&lcdInitSeq[x])!=0) {
		ctr=pgm_read_byte(&lcdInitSeq[x++]); //Get amount of bytes in command
		sendCmdLcd();
		while (ctr!=0) { //Send that amount of bytes to LCD
			sendByteLcd(pgm_read_byte(&lcdInitSeq[x++]));
			ctr--;
		}
		endCmdLcd();
		_delay_ms(20);
	}
	//Reset internal variables & clear lcd
	xpos=3; 
	ypos=0; 
	fgcol=0xfff;
	bgcol=0;
	clearLcd();
    stdout = &mystdout;
}

void clearLcd() {
	int x;
	windowLcd(0,0,104,80);
	sendDataLcd();
	//Fill lcd with black.
	for (x=0; x<104*80; x++) {
		sendpixel(0);
	}
	endCmdLcd();
}

//To draw on the LCD, you first have to define a window where your
//data will go to.
void windowLcd(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2) {
	x1=x1*2; 
	x2=x2*2+1;
	sendCmdLcd();
	sendByteLcd(0xF0);
	sendByteLcd(0x00|(x1&0xf));
	sendByteLcd(0x10|(x1>>4));
	sendByteLcd(0x20|(y1&0xf));
	sendByteLcd(0x30|(y1>>4));
	endCmdLcd();
	sendCmdLcd();
	sendByteLcd(0xF5);
	sendByteLcd(0x00|(x2&0xf));
	sendByteLcd(0x10|(x2>>4));
	sendByteLcd(0x20|(y2&0xf));
	sendByteLcd(0x30|(y2>>4));
	endCmdLcd();
}
